// UC: 21020 - Computação Gráfica (2022/23)
// Aluno: 1902551 - Pedro Gaspar

/**
 * Algoritmo de Bresenham
 * @param {{x: number; y: number}} p Coordenada de início
 * @param {{x: number; y: number}} q Coordenada de fim
 * @returns {{x: number; y: number}[]} Array de coordenadas da interseção
 * @see https://en.wikipedia.org/wiki/Bresenham%27s_line_algorithm
 */
export function lineMP(p, q) {
  // Distância horizontal entre os pontos (dx: Distance X)
  const dx = Math.abs(q.x - p.x);

  // Verifica o declive horizontal, se sobe ou desce (xs: Slope X)
  const sx = p.x < q.x ? 1 : -1;

  // Distância vertical entre os pontos (dy: Distance Y)
  const dy = Math.abs(q.y - p.y);

  // Verifica o declive vertical, se sobe ou desce (sy: Slope Y)
  const sy = p.y < q.y ? 1 : -1;

  // O algoritmo de Bresenham é um "incremental error algorithm", ou seja,
  // utiliza apenas somas e subtrações para atualizar um termo de erro, em vez
  // de utilizar divisões e/ou multiplicações, que são mais muito mais pesadas
  // em termos de performance.
  let err = dx - dy;

  // Coordenadas resultantes
  const coords = [];

  while (true) {
    // Colocar o ponto atual no array
    coords.push({ x: p.x, y: p.y });

    if (p.x === q.x && p.y === q.y) {
      // Chegámos ao ponto final
      break;
    }

    const e2 = err * 2;

    if (e2 > -dy) {
      err -= dy;
      p.x += sx;
    }

    if (e2 < dx) {
      err += dx;
      p.y += sy;
    }
  }

  return coords;
}
